package org.gcube.application.cms.plugins.reports;

import com.fasterxml.jackson.core.JsonProcessingException;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.bson.Document;
import org.gcube.application.cms.plugins.faults.PluginExecutionException;
import org.gcube.application.cms.plugins.requests.BaseExecutionRequest;
import org.gcube.application.cms.serialization.Serialization;
import org.gcube.application.geoportal.common.model.document.ProfiledDocument;
import org.gcube.application.geoportal.common.model.document.lifecycle.LifecycleInformation;


@Data
@Slf4j
public class DocumentHandlingReport<T extends BaseExecutionRequest> extends Report{

    T theRequest;
    Document resultingDocument;
    LifecycleInformation toSetLifecycleInformation;

    public ProfiledDocument prepareResult() throws JsonProcessingException, PluginExecutionException {
        log.trace("Preparing document ID {} from report ... ",theRequest.getDocument().get_id());
        this.validate();
        ProfiledDocument toReturn = theRequest.getDocument();
        if(resultingDocument !=null ) {
            log.trace("Updating Document ");
            toReturn.setTheDocument(resultingDocument);
        }
        if(toSetLifecycleInformation != null){
            log.trace("Merging lifecycleInformation");
//            Document original = Serialization.asDocument(toReturn.getLifecycleInformation());
//            Document toSet = Serialization.asDocument(toSetLifecycleInformation);
//            toSet.putAll(original);
//            toReturn.setLifecycleInformation(Serialization.convert(toSet,LifecycleInformation.class));
            toReturn.setLifecycleInformation(toSetLifecycleInformation);
        }

        // Force Report status into info
        log.trace("Report status is {} ",getStatus());
        LifecycleInformation info = toReturn.getLifecycleInformation();
        switch (getStatus()){
            case ERROR: {
                info.setLastOperationStatus(LifecycleInformation.Status.ERROR);
                this.getMessages().forEach(s -> info.addErrorMessage(s));
                break;
            }
            case WARNING:{
                info.setLastOperationStatus(LifecycleInformation.Status.WARNING);
                this.getMessages().forEach(s -> info.addWarningMessage(s));
                break;
            }
            case OK: {
                if(info.getLastOperationStatus()==null)
                    info.setLastOperationStatus(LifecycleInformation.Status.OK);
                break;
            }
        }
        return toReturn;
    }
}
